/* ====================================================================
 * FTD4Linux Software License, Version 1.0
 *
 * Copyright (c) 2000-2004 The FTD4Linux Development Team.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        FTD4Linux Development Team (http://www.ftd4linux.2y.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "FTD" and "FTD4Linux" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact alguno@ftd4linux.2y.net.
 *
 * 5. Products derived from this software may not be called "FTD4Linux",
 *    nor may "FTD4Linux" appear in their name, without prior written
 *    permission of the FTD4Linux Development Team.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE FTD4LINUX DEVELOPMENT TEAM OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _FTD_GUI_CENTER_IRC_H_
#define _FTD_GUI_CENTER_IRC_H_

struct _ftd_gui_center_irc {
    GtkWidget       *widget;
    GtkWidget       *cmdclose;
    GtkListStore    *storeUsers;
    GtkWidget       *treeviewUsers;
    GtkWidget       *lblSubject;
    GtkWidget       *txtChat;
    GtkWidget       *btnConnect;
    GtkWidget       *lblConnect;
    GtkWidget       *imgConnect;
    GtkWidget       *entryChat;
};

void         ftd_gui_center_irc_new(void);

typedef enum {
    IRC_STATE_DISCONNECTED,
    IRC_STATE_CONNECTING,
    IRC_STATE_CONNECTED
} FtdGuiIRCState;

typedef struct _ftd_gui_irc FtdGuiIRC;

struct _ftd_gui_irc {
    GIOChannel *io_channel;     // Blijf van deze 2 variabelen af! Deze worden
    FtdGuiIRCState state;       // gebruikt in het interne deel van de IRC client

    const char *nickname;
    const char *altNickname;
    const char *channel;
    const char *server;
    gboolean isUsingAltNickname;

    gpointer data;

    void (*state_changed_cb) (FtdGuiIRC *irc_data, FtdGuiIRCState old_state, const char *errmsg, gpointer data);
    void (*on_msg_received_cb) (FtdGuiIRC *irc_data, const char *source, const char *msg, gboolean is_action, gboolean is_privmsg, gpointer data);
    void (*on_join_cb) (FtdGuiIRC *irc_data, const FtdList *users, const char *topic, gpointer data);
    void (*on_user_join_part_cb) (FtdGuiIRC *irc_data, const char *username, const char *channel, const char *reason, gboolean has_joined);
    void (*on_mode_change_cb) (FtdGuiIRC *irc_data, const char *change_performed_by, const char *username, char *mode);
    void (*on_nick_change_cb) (FtdGuiIRC *irc_data, const char *old_nickname, const char *new_nickname);
};

void         ftd_gui_irc_send_msg(FtdGuiIRC *irc_data, const char *dest, const char *msg, gboolean is_action);
gboolean     ftd_gui_irc_connect(FtdGuiIRC *irc_data);
void         ftd_gui_irc_disconnect(FtdGuiIRC *irc_data);

#endif /* _FTD_GUI_CENTER_IRC_H_ */
