/* ====================================================================
 * FTD4Linux Software License, Version 1.0
 *
 * Copyright (c) 2000-2004 The FTD4Linux Development Team.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        FTD4Linux Development Team (http://www.ftd4linux.2y.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "FTD" and "FTD4Linux" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact alguno@ftd4linux.2y.net.
 *
 * 5. Products derived from this software may not be called "FTD4Linux",
 *    nor may "FTD4Linux" appear in their name, without prior written
 *    permission of the FTD4Linux Development Team.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE FTD4LINUX DEVELOPMENT TEAM OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "ftd_gui_main.h"
#include "ftd_gui_worksheet.h"
#include "ftd_gui_center.h"
#include "ftd_gui_center_eventlist.h"
#include "ftd_gui_errhandler.h"
#include "ftd_gui_pixbuf.h"
#include "ftd_gui_xslt.h"
#include "ftd_gui_browser.h"
#include "ftd_gui_parse.h"

void ftd_gui_center_eventlist_row_activated(GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *column, gpointer user_data);

void
ftd_gui_center_eventlist_new(void)
{
#ifdef NO_EXTERNAL_BROWSER
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;
#endif
    GtkWidget *label;
    FtdGuiCenterEventList *eventlist = (FtdGuiCenterEventList *) g_malloc0(sizeof(FtdGuiCenterEventList));
    g_assert(gui);
    g_assert(gui->worksheet);
    gui->worksheet->center->eventlist = eventlist;

#ifdef NO_EXTERNAL_BROWSER
    eventlist->widget = gtk_scrolled_window_new (NULL, NULL);

    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (eventlist->widget),
                                    GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

    eventlist->store = gtk_list_store_new(8,   G_TYPE_INT,         /* eventid */
                                               (sizeof(time_t) == 4) ? G_TYPE_INT : G_TYPE_INT64,         /* stamp */
                                               GDK_TYPE_PIXBUF,    /* status */
                                               GDK_TYPE_PIXBUF,    /* cat */
                                               G_TYPE_INT,         /* scat */
                                               G_TYPE_STRING,      /* title */
                                               G_TYPE_STRING,      /* poster */
                                               G_TYPE_POINTER);    /* event */

    eventlist->browser = gtk_tree_view_new_with_model(GTK_TREE_MODEL(eventlist->store));

    renderer = gtk_cell_renderer_pixbuf_new ();
    column = gtk_tree_view_column_new_with_attributes ("S",
                                                    renderer,
                                                    "pixbuf", 2,
                                                    NULL);
    gtk_tree_view_column_set_min_width (column, 25);
    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
    gtk_tree_view_column_set_resizable (column, FALSE);
    gtk_tree_view_append_column (GTK_TREE_VIEW (eventlist->browser), column);

    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes ("Datum",
                                                    renderer,
                                                    "text", 1,
                                                    NULL);
    gtk_tree_view_column_set_min_width (column, 100);
    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
    gtk_tree_view_column_set_resizable (column, FALSE);
    gtk_tree_view_append_column (GTK_TREE_VIEW (eventlist->browser), column);

    renderer = gtk_cell_renderer_pixbuf_new ();
    column = gtk_tree_view_column_new_with_attributes ("Cat",
                                                    renderer,
                                                    "pixbuf", 3,
                                                    NULL);
    gtk_tree_view_column_set_min_width (column, 40);
    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
    gtk_tree_view_column_set_resizable (column, FALSE);
    gtk_tree_view_append_column (GTK_TREE_VIEW (eventlist->browser), column);

    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes ("Title",
                                                    renderer,
                                                    "text", 5,
                                                    NULL);
    gtk_tree_view_column_set_min_width (column, 100);
    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_GROW_ONLY);
    gtk_tree_view_column_set_resizable (column, TRUE);
    gtk_tree_view_column_set_expand(column, TRUE);
    gtk_tree_view_append_column (GTK_TREE_VIEW (eventlist->browser), column);

    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes ("Poster",
                                                    renderer,
                                                    "text", 6,
                                                    NULL);
    gtk_tree_view_column_set_min_width (column, 100);
    gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_GROW_ONLY);
    gtk_tree_view_column_set_resizable (column, TRUE);
    gtk_tree_view_column_set_expand(column, TRUE);
    gtk_tree_view_append_column (GTK_TREE_VIEW (eventlist->browser), column);

    gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW(eventlist->widget), eventlist->browser);

    gtk_widget_show(eventlist->browser);
    gtk_widget_show(eventlist->widget);

    gtk_container_add (GTK_CONTAINER (gui->worksheet->center->widget), gui->worksheet->center->eventlist->widget);

    label = gtk_label_new ("FTD Postings");
    gtk_widget_show (label);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (gui->worksheet->center->widget),
                                gtk_notebook_get_nth_page (
                                    GTK_NOTEBOOK (gui->worksheet->center->widget), -1), label);
    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);

    g_signal_connect(G_OBJECT(eventlist->browser), "row-activated", G_CALLBACK(ftd_gui_center_eventlist_row_activated), NULL);
#else
    eventlist->browser_data = ftd_gui_browser_new(gui->worksheet->center->widget, FALSE);

    label = gtk_label_new ("Postoverzicht");
    gtk_widget_show (label);
    gtk_notebook_set_tab_label (GTK_NOTEBOOK (gui->worksheet->center->widget),
                                gtk_notebook_get_nth_page (
                                    GTK_NOTEBOOK (gui->worksheet->center->widget), -1), label);
    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);

    /* Signal handler om het statusvenstertje te vernietigen wanneer
     * het laden van de HTML pagina klaar is */
    ftd_gui_browser_set_load_finished_cb(eventlist->browser_data, ftd_gui_center_eventlist_load_finished, NULL);

    /* Signal handler om de url klikjes vanuit het componentje op te vangen */
    ftd_gui_browser_set_link_clicked_cb(eventlist->browser_data, ftd_gui_center_eventlist_open_uri, NULL);

    gtk_notebook_set_current_page (GTK_NOTEBOOK (gui->worksheet->center->widget), -1);
#endif /* NO_EXTERNAL_BROWSER */
}

#ifdef NO_EXTERNAL_BROWSER
static gboolean
eventlist_show_func(gpointer key, gpointer value, gpointer data)
{
    GdkPixbuf *status_image = NULL;
    GdkPixbuf *cat_image = NULL;
    FtdEvent *event = (FtdEvent *) value;
    GtkTreeIter iter;
    char *tmp;

    switch(event->status) {
        case FTDSTATUS_DELETED:
            status_image = ftd_gui_main_get_pixbuf("deleted");
            break;
        case FTDSTATUS_SPOT:
            status_image = ftd_gui_main_get_pixbuf("spot");
            break;
        case FTDSTATUS_READY:
            status_image = ftd_gui_main_get_pixbuf("ready");
            break;
        case FTDSTATUS_ANNOUNCE:
            status_image = ftd_gui_main_get_pixbuf("announce");
            break;
        case FTDSTATUS_UPLOADING:
            status_image = ftd_gui_main_get_pixbuf("uploading");
            break;
        case FTDSTATUS_UNSUPPORTED:
            status_image = ftd_gui_main_get_pixbuf("unsupported");
            break;
        default:
            break;
    }

    tmp = g_strdup_printf("ct%i%i", event->cat, event->sCat);
    cat_image = ftd_gui_main_get_pixbuf(tmp);
    g_free(tmp);

    gtk_list_store_append(gui->worksheet->center->eventlist->store, &iter);
    gtk_list_store_set(gui->worksheet->center->eventlist->store, &iter, 0, event->postId,
                                                                        1, event->stamp,
                                                                        2, status_image,
                                                                        3, cat_image,
                                                                        4, event->sCat,
                                                                        5, ftd_string_value(event->title),
                                                                        6, ftd_string_value(event->poster),
                                                                        7, event,
                                                                        -1);

    return FALSE;
}
#endif

gboolean
ftd_gui_center_eventlist_show(gpointer data)
{
#ifdef NO_EXTERNAL_BROWSER
    FtdTree *eventTree;
#endif
    int pagenum;

    pagenum = gtk_notebook_page_num(GTK_NOTEBOOK(gui->worksheet->center->widget), ftd_gui_browser_get_widget(gui->worksheet->center->eventlist->browser_data));
    gtk_notebook_set_current_page(GTK_NOTEBOOK(gui->worksheet->center->widget), pagenum);

#ifdef NO_EXTERNAL_BROWSER
    gtk_list_store_clear(gui->worksheet->center->eventlist->store);

    eventTree = ftd_events_tree_filtered_get();

    ftd_tree_foreach(eventTree, eventlist_show_func, NULL);

    ftd_gui_center_eventlist_load_finished(NULL, NULL);
#else
    ftd_gui_browser_navigate(gui->worksheet->center->eventlist->browser_data, HTML_LIST_FILE);
#endif

    return FALSE;
}

void
ftd_gui_center_eventlist_load_finished (GtkWidget *caller, gpointer data)
{
    g_assert (gui);

    ftd_gui_status_push();

    gtk_statusbar_push(GTK_STATUSBAR(gui->statusbar), 0, "Klaar...");
}

static void
forum_fetch_cb(FtdErrorCode retval, int posting, time_t lastCheck, char *errmsg, gpointer data)
{
    FtdGuiCenterEventInfo *eventinfo = (FtdGuiCenterEventInfo *) data;

    if (retval != ERROR_NONE) {
        ftd_gui_errhandler(gui->window, FTD_ONDERDEEL_EVENTINFO_FORUM, retval, errmsg);
        ftd_gui_center_eventinfo_sluiten(eventinfo->cmdclose, NULL);
        return;
    }

#ifndef NO_EXTERNAL_BROWSER
    retval = ftd_gui_center_eventinfo_process(posting);
    if (retval != ERROR_NONE) {
        ftd_gui_errhandler(gui->window, FTD_ONDERDEEL_EVENTINFO_FORUM, retval, NULL);
        ftd_gui_center_eventinfo_sluiten(eventinfo->cmdclose, NULL);
        return;
    }
#endif

    // Toon de gegevens op het scherm
    ftd_gui_center_eventinfo_show(ftd_gui_browser_get_widget(eventinfo->browser_data), NULL);
}

static void
open_posting_fase2(FtdErrorCode retval, int posting, char *errmsg, gpointer data)
{
    FtdGuiCenterEventInfo *eventinfo = (FtdGuiCenterEventInfo *) data;
    FtdEvent *event;
    FTDEventlist *eventlist;

    if (retval != ERROR_NONE) {
        ftd_gui_errhandler(gui->window, FTD_ONDERDEEL_EVENTINFO, retval, errmsg);
        ftd_gui_center_eventinfo_sluiten(eventinfo->cmdclose, NULL);
        return;
    }

    // Stel de titel van het tabblad goed in
    eventlist = ftd_eventlist_get();
    event = ftd_eventlist_lookup_id(eventlist, posting);
    g_object_unref(eventlist);

    if (event && event->title) {
        char bericht[64];
        FtdString *tmp;

        tmp = ftd_string_replace(event->title, "&amp;", "&");
        if (ftd_string_length(tmp) > 20) {
            char title[21];
            strncpy(title, ftd_string_value(tmp), 21); // Dit levert onder Win32 een truncate melding op, maar dat is ook de bedoeling in deze situatie
            title[20] = '\0';
            snprintf(bericht, sizeof(bericht), "%s...", title);
        } else {
            snprintf(bericht, sizeof(bericht), "%s", ftd_string_value(tmp));
        }

        ftd_event_free(event);
        event = NULL;

        gtk_label_set_text(GTK_LABEL(eventinfo->label), bericht);
    }

    // Haal alle bijbehorende forum reacties op
    ftd_events_forum_fetch_async(posting, (time_t) 0, forum_fetch_cb, data);
}

static void
open_posting(int posting)
{
    FtdErrorCode retval;
    char *errmsg;
    FtdGuiCenterEventInfo *eventinfo;

    // Open een tabblad voor de postinfo in te dumpen
    eventinfo = ftd_gui_center_eventinfo_new(posting);

    if (!eventinfo) {
        return;
    }

    /* Haal de gegevens van het FTD event op. */
    ftd_eventinfo_fetch_async(posting, open_posting_fase2, eventinfo);
}

gboolean
ftd_gui_center_eventlist_open_uri (GtkWidget *embed, const char *uri, gpointer data)
{
    char *begin;

    g_assert (gui);

    /* =======================
     * Externe URL's (#link)
     * =======================
     */
    begin = strstr(uri, "#link");
    if (begin) {
        begin += 5;

        ftd_gui_center_launch_browser(begin);

        return TRUE;
    }

    /* =======================
     * #post
     * =======================
     */
    begin = strstr(uri, "#post");
    if (begin) {
        begin += 5;

        open_posting(atoi(begin));

        return TRUE;
    }

    /* ========================
     * #sortmode
     * ========================
     */
    begin = strstr(uri, "#sortmode");
    if (begin) {
         /* Laad gegevens uit de posts cache, sorteer deze en bewaar het als HTML_LIST_FILE */
        int sortmode;
        int ret;
        FtdFilter *filter;
        FTDEventlist *eventlist;

        eventlist = ftd_eventlist_get();
        filter = ftd_eventlist_get_filter(eventlist);

#ifndef NO_EXTERNAL_BROWSER
        FtdString *uitvoer;
#endif

        begin += 9;
        sortmode = atoi(begin);

        switch (sortmode) {
            case 0:
                if (filter->veld == FTDEVENT_SORT_DATUM) {
                    filter->asc = !filter->asc;
                } else {
                    filter->asc = FALSE;   // Sorteer standaard met meest recente item eerst
                }

                filter->veld = FTDEVENT_SORT_DATUM;

                break;
            case 1:
                if (filter->veld == FTDEVENT_SORT_TITEL) {
                    filter->asc = !filter->asc;
                } else {
                    filter->asc = TRUE;
                }

                filter->veld = FTDEVENT_SORT_TITEL;

                break;
            case 2:
                if (filter->veld == FTDEVENT_SORT_POSTER) {
                    filter->asc = !filter->asc;
                } else {
                    filter->asc = TRUE;
                }

                filter->veld = FTDEVENT_SORT_POSTER;

                break;
            case 5:
                if (filter->veld == FTDEVENT_SORT_RATING) {
                    filter->asc = !filter->asc;
                } else {
                    filter->asc = TRUE;
                }

                filter->veld = FTDEVENT_SORT_RATING;

                break;
            default:
                g_warning("ftd_gui_center_eventlist_open_uri(): Onbekende sortmode :%i\n", sortmode);
                break;
        }

        ftd_eventlist_apply_filter(eventlist, filter);
        g_object_unref(eventlist);

#ifndef NO_EXTERNAL_BROWSER
        uitvoer = ftd_events_sort_filtered(ftd_gui_parse);

        if (filter->categoriseerd) {
            ret = ftd_gui_xslt_process (uitvoer, XSLT_TYPE_LIST, HTML_LIST_FILE);
        } else {
            ret = ftd_gui_xslt_process (uitvoer, XSLT_TYPE_LIST_TOTAL, HTML_LIST_FILE);
        }

        ftd_filter_free_filter(filter);

        ftd_string_free (uitvoer);

        if (ret != 0) {
            g_warning("ftd_gui_center_eventlist_open_uri(): verwerken van xslt sheet mislukt\n");
            return TRUE;
        }
#endif

        ftd_gui_center_eventlist_show(NULL);

        return TRUE;
    }

    /* =======================
     * #info
    * =======================
    */
    begin = strstr(uri, "#info");
    if (begin) {
        ftd_gui_dialog_show(NULL, "Not yet implemented", gui->window, TRUE);
        return TRUE;
    }

    /* =======================
     * #add
     * =======================
    */
    begin = strstr(uri, "#add");
    if (begin) {
        int postid;
        begin += 4;
        postid = atoi(begin);

        if (ftd_watchlist_is_in_list(postid)) {
            int teller;
            char * path_tmp = NULL;
            FtdString * path_tmp2 = NULL;
            FtdString * path_tmp3 = NULL;
            FtdString * path_tmp4 = NULL;
            char * path = NULL;
            char buf[2048];
            char buf2[2048];

            // Postid is reeds aanwezig in de watchlist -> verwijderen
            ftd_watchlist_remove(postid);

            // Pas het watchlist icoontje in het postoverzicht aan
            path_tmp = ftd_file_parse_data_dir(NULL);
            path_tmp2 = ftd_string_new(path_tmp);
            path_tmp3 = ftd_string_replace(path_tmp2, " ", "%20");
            path_tmp4 = ftd_string_replace(path_tmp3, "\\", "/");
            path = ftd_string_value(path_tmp4);
            snprintf(buf, sizeof(buf), "file://%s/images/dlb1a.bmp", path);
            ftd_string_free(path_tmp4);
            ftd_string_free(path_tmp3);
            ftd_string_free(path_tmp2);
            g_free(path_tmp);

            snprintf(buf2, sizeof(buf2), "p%d", postid);
            ftd_gui_browser_set_element_attribute(gui->worksheet->center->eventlist->browser_data, buf2, "src", buf);


            // Pas het watchlist icoontje in een eventueel geopende postinfo aan
            path_tmp = ftd_file_parse_data_dir(NULL);
            path_tmp2 = ftd_string_new(path_tmp);
            path_tmp3 = ftd_string_replace(path_tmp2, " ", "%20");
            path_tmp4 = ftd_string_replace(path_tmp3, "\\", "/");
            path = ftd_string_value(path_tmp4);
            snprintf(buf, sizeof(buf), "file://%s/images/b4-1.bmp", path);
            ftd_string_free(path_tmp4);
            ftd_string_free(path_tmp3);
            ftd_string_free(path_tmp2);
            g_free(path_tmp);

            if( gui->worksheet->center->eventinfo ) {
                for( teller = 0; teller < MAX_EVENT_INFO; teller++ ) {
                    if( gui->worksheet->center->eventinfo[teller] && gui->worksheet->center->eventinfo[teller]->post_id == postid ) {
                        ftd_gui_browser_set_element_attribute( gui->worksheet->center->eventinfo[teller]->browser_data, "dlbut", "src", buf );
                        break;
                    }
                }
            }
        } else {
            int teller;
            char * path_tmp = NULL;
            FtdString * path_tmp2 = NULL;
            FtdString * path_tmp3 = NULL;
            FtdString * path_tmp4 = NULL;
            char * path = NULL;
            char buf[2048];
            char buf2[2048];

            ftd_watchlist_add(postid);

            // Pas het watchlist icoontje in het postoverzicht aan
            path_tmp = ftd_file_parse_data_dir(NULL);
            path_tmp2 = ftd_string_new(path_tmp);
            path_tmp3 = ftd_string_replace(path_tmp2, " ", "%20");
            path_tmp4 = ftd_string_replace(path_tmp3, "\\", "/");
            path = ftd_string_value(path_tmp4);
            snprintf(buf, sizeof(buf), "file://%s/images/dlb2a.bmp", path);
            ftd_string_free(path_tmp4);
            ftd_string_free(path_tmp3);
            ftd_string_free(path_tmp2);
            g_free(path_tmp);

            snprintf(buf2, sizeof(buf2), "p%d", postid);
            ftd_gui_browser_set_element_attribute(gui->worksheet->center->eventlist->browser_data, buf2, "src", buf);

            // Pas het watchlist icoontje in een eventueel geopende postinfo aan
            path_tmp = ftd_file_parse_data_dir(NULL);
            path_tmp2 = ftd_string_new(path_tmp);
            path_tmp3 = ftd_string_replace(path_tmp2, " ", "%20");
            path_tmp4 = ftd_string_replace(path_tmp3, "\\", "/");
            path = ftd_string_value(path_tmp4);
            snprintf(buf, sizeof(buf), "file://%s/images/b4-3.bmp", path);
            ftd_string_free(path_tmp4);
            ftd_string_free(path_tmp3);
            ftd_string_free(path_tmp2);
            g_free(path_tmp);

            if( gui->worksheet->center->eventinfo ) {
                for( teller = 0; teller < MAX_EVENT_INFO; teller++ ) {
                    if( gui->worksheet->center->eventinfo[teller] && gui->worksheet->center->eventinfo[teller]->post_id == postid ) {
                        ftd_gui_browser_set_element_attribute( gui->worksheet->center->eventinfo[teller]->browser_data, "dlbut", "src", buf );
                        break;
                    }
                }
            }
        }

        return TRUE;
    }

    return FALSE;
}

#ifdef NO_EXTERNAL_BROWSER
void
ftd_gui_center_eventlist_row_activated(GtkTreeView *treeview, GtkTreePath *path, GtkTreeViewColumn *column, gpointer user_data)
{
    GtkTreeIter iter;
    int postId = -1;
    char *url;

    g_assert(gui);

    if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(gui->worksheet->center->eventlist->store), &iter, path)) {
        g_print("ftd_gui_center_eventlist_row_activated(): gtk_tree_model_get_iter() mislukt\n");
        return;
    }

    gtk_tree_model_get(GTK_TREE_MODEL(gui->worksheet->center->eventlist->store), &iter, 0, &postId, -1);

    url = g_strdup_printf("#post%i", postId);
    ftd_gui_center_eventlist_open_uri(NULL, url, NULL);
    g_free(url);
}
#endif
