#include <stdio.h>
#ifndef _MSC_VER
#include <unistd.h>
#endif
#include <errno.h>
#include <sys/types.h>
#ifdef WIN32
#include <winsock2.h>
#include <ws2tcpip.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>    /* sockaddr_in structure */
#include <netdb.h>          /* /etc/hosts table entries */
#endif
#include <string.h>
#include <ctype.h>
#include <glib.h>

#include "ftd_gui_main.h"
#include "ftd_gui_center_irc.h"

#define DEBUG 0

static char *strip_color (char *text)
{
    int nc = 0;
    int i = 0;
    int col = 0;
    int len = strlen (text);
    char *new_str = g_malloc (len + 2);

    while (len > 0)
    {
        if ((col && isdigit (*text) && nc < 2) ||
            (col && *text == ',' && isdigit (*(text+1)) && nc < 3)) {

            nc++;
            if (*text == ',') {
                nc = 0;
            }
        } else {
            col = 0;
            switch (*text) {
                case '\003':                      /*ATTR_COLOR: */
                    col = 1;
                    nc = 0;
                    break;

                case '\007':                      /*ATTR_BEEP: */
                case '\017':                      /*ATTR_RESET: */
                case '\026':                      /*ATTR_REVERSE: */
                case '\002':                      /*ATTR_BOLD: */
                case '\037':                      /*ATTR_UNDERLINE: */
                    break;

                default:
                    new_str[i] = *text;
                    i++;
            }
        }

        text++;
        len--;
    }

    new_str[i] = 0;

    return new_str;
}

static gboolean
parse_irc_message (GIOChannel *channel, GIOCondition condition, gpointer data)
{
    FtdGuiIRC *irc_data = (FtdGuiIRC *) data;
    char *response;
    char niks[1024];
    char PONG[10];
    char msg[4096];
    char nickname[256];
    char channel_name[256];
    char mode[256];
    int ret = 0;
    int num;

    char *regel = NULL;
    GError *err = NULL;

    memset(msg, 0, sizeof(msg));

    if ((condition & G_IO_IN) != G_IO_IN) {
        // Er ging iets niet goed...disconnecten maar
#ifdef DEBUG
        g_print("condition = %i\n", condition);
#endif
        ftd_gui_irc_disconnect(irc_data);
        return FALSE;
    }

    if (g_io_channel_read_line(channel, &regel, NULL, NULL, &err) != G_IO_STATUS_NORMAL) {
        // Er ging iets niet goed tijdens het lezen vanuit de socket, disconnecten maar
#ifdef DEBUG
        if (err) {
            g_print(__FILE__ ":%i %s\n", __LINE__, err->message);
            g_error_free(err);
        } else {
            g_print("huh? errmsg = %s\n", strerror(errno));
        }
#endif

        g_io_channel_shutdown(channel, FALSE, NULL);
        return FALSE;
    }

    if (regel[strlen(regel) - 1] == '\n') {
        regel[strlen(regel) -1] = '\0';
    }

    if (regel[strlen(regel) - 1] == '\r') {
        regel[strlen(regel) - 1] = '\0';
    }

#ifdef DEBUG
    g_print("received data = %s\n", regel);
#endif

    ret = sscanf(regel, "%*s %i %*s", &num);
    if (ret == 1) {
        static char *subject = NULL;
        static FtdList *users = NULL;
        gchar **values;
        int i;
        FtdList *list;

        response = NULL;

        switch(num) {
            case 001:
                // :terneuzen.irc.ftd4linux.nl 001 Alguno_FTD4Linux :Welcome to the FTD4Linux IRC Network
                // We kunnen joinen
                response = g_strdup_printf("JOIN %s\r\n", irc_data->channel);
                break;

            case 332:
                // :terneuzen.irc.ftd4linux.nl 332 Alguno_FTD4Linux #tlx :Dit is een subject
                ret = sscanf(regel, "%*s %*i %*s %*s :%255[^\n]", &msg);
                subject = g_strdup(msg);
                break;

            case 353:
                // :terneuzen.irc.ftd4linux 353 Alguno_FTD4Linux = #tlx :Alguno_FTD4Linux Squall[PG] |Marnux| @Icheb n0ne X @Alguno FrodoBaggins slazh @FTD_Bot
                ret = sscanf(regel, "%*s %*i %*s = %*s :%4095[^\n]", &msg);
                values = g_strsplit(msg, " ", 0);
                i = 0;
                while (values[i]) {
                    if (strlen(values[i]) > 0) {
                        users = ftd_list_append(users, g_strdup(values[i]));
                    }

                    i++;
                }
                g_strfreev(values);
                break;

            case 366:
                // :terneuzen.irc.ftd4linux 366 Alguno_FTD4Linux #tlx: End of /NAMES list.
                if (irc_data->on_join_cb) {
                    irc_data->on_join_cb(irc_data, users, subject, irc_data->data);
                }

                list = users;
                while (list) {
                    g_free(ftd_list_value(list));
                    list = ftd_list_next(list);
                }
                ftd_list_free(users);
                users = NULL;

                g_free(subject);
                subject = NULL;

                irc_data->state = IRC_STATE_CONNECTED;
                if (irc_data->state_changed_cb) {
                    irc_data->state_changed_cb(irc_data, IRC_STATE_CONNECTING, NULL, irc_data->data);
                }

                break;

            case 433:   // Nickname is already in use
                if (!irc_data->isUsingAltNickname) {
                    irc_data->isUsingAltNickname = TRUE;
                    response = g_strdup_printf("NICK %s\n", irc_data->altNickname);
                } else {
                    irc_data->state = IRC_STATE_DISCONNECTED;
                    if (irc_data->state_changed_cb) {
                        regel = g_strdup_printf("Nicknames '%s' en '%s' zijn reeds in gebruik", irc_data->nickname, irc_data->altNickname);
                        irc_data->state_changed_cb(irc_data, IRC_STATE_CONNECTING, regel, irc_data->data);
                        g_free(regel);
                    }
                }

                break;
        }

        if (response) {
#ifdef DEBUG
            printf("response = %s\n", response);
#endif

            g_io_channel_write_chars(irc_data->io_channel, response, -1, NULL, NULL);
            g_io_channel_flush(irc_data->io_channel, NULL);
            g_free(response);
        }
    }

// PING :14CE04
    ret = sscanf(regel, "PING :%s", &PONG);
    if (ret == 1) {
        // Ping-Pong effect
        response = g_strdup_printf("PONG :%s\r\n", PONG);

#ifdef DEBUG
        printf("response = %s\n", response);
#endif

        // En versturen maar
        g_io_channel_write_chars(channel, response, -1, NULL, NULL);
        g_io_channel_flush(channel, NULL);
        g_free(response);
    }

// :Alguno!me@ftd4linux.2y.net JOIN :#tlx
    memset(nickname, 0, sizeof(nickname));
    memset(msg, 0, sizeof(msg));
    ret = sscanf(regel, "%s JOIN :%s[^\n]", &nickname, &channel_name);
    if (ret == 2) {
        char *tmp;
        int i, len;
        char *clean_msg;

        // Sloop de nickname
        // :Alguno!me@ftd4linux.xs4all.nl
        tmp = strstr(nickname, "!");
        if (tmp) {
            len = tmp - nickname;
            for (i = 1; i < len; i++) {
                nickname[i - 1] = nickname[i];
            }
            nickname[len - 1] = '\0';
        }

        if (irc_data->on_user_join_part_cb) {
            irc_data->on_user_join_part_cb(irc_data, nickname, channel_name, NULL, TRUE);
        }
    }

// :Alguno!me@ftd4linux.2y.net PART #tlx :Ik ga weg
    memset(nickname, 0, sizeof(nickname));
    memset(msg, 0, sizeof(msg));
    ret = sscanf(regel, "%s PART %s :%255[^\n]", &nickname, &channel_name, &msg);
    if (ret == 3) {
        char *tmp;
        int i, len;
        char *clean_msg;

        // Sloop de nickname
        // :Alguno!me@ftd4linux.xs4all.nl
        tmp = strstr(nickname, "!");
        if (tmp) {
            len = tmp - nickname;
            for (i = 1; i < len; i++) {
                nickname[i - 1] = nickname[i];
            }
            nickname[len - 1] = '\0';
        }

        if (irc_data->on_user_join_part_cb) {
            irc_data->on_user_join_part_cb(irc_data, nickname, channel_name, msg, FALSE);
        }
    }

// :Alguno!me@ftd4linux.2y.net QUIT :Quit:
    memset(nickname, 0, sizeof(nickname));
    memset(msg, 0, sizeof(msg));
    ret = sscanf(regel, "%s QUIT :%255[^\n]", &nickname, &msg);
    if (ret == 3) {
        char *tmp;
        int i, len;
        char *clean_msg;

        // Sloop de nickname
        // :Alguno!me@ftd4linux.xs4all.nl
        tmp = strstr(nickname, "!");
        if (tmp) {
            len = tmp - nickname;
            for (i = 1; i < len; i++) {
                nickname[i - 1] = nickname[i];
            }
            nickname[len - 1] = '\0';
        }

        if (irc_data->on_user_join_part_cb) {
            irc_data->on_user_join_part_cb(irc_data, nickname, NULL, msg, FALSE);
        }
    }

// :Alguno!me@ftd4linux.2y.net NICK :Alguno-laptop
    memset(nickname, 0, sizeof(nickname));
    memset(msg, 0, sizeof(msg));
    ret = sscanf(regel, "%s NICK :%255[^\n]", &nickname, &msg);
    if (ret == 2) {
        char *tmp;
        int i, len;
        char *clean_msg;

        // Sloop de nickname
        // :Alguno!me@ftd4linux.xs4all.nl
        tmp = strstr(nickname, "!");
        if (tmp) {
            len = tmp - nickname;
            for (i = 1; i < len; i++) {
                nickname[i - 1] = nickname[i];
            }
            nickname[len - 1] = '\0';
        }

        if (irc_data->on_nick_change_cb) {
            irc_data->on_nick_change_cb(irc_data, nickname, msg);
        }
    }

// :FTD_Bot!FTD_Bot@localhost MODE #tlx +o Alguno
    memset(nickname, 0, sizeof(nickname));
    memset(msg, 0, sizeof(msg));
    memset(mode, 0, sizeof(mode));
    ret = sscanf(regel, "%s MODE %s %s %255[^\n]", &nickname, &channel_name, &mode, &msg);
    if (ret == 4) {
        char *tmp;
        int i, len;
        char *clean_msg;

        // Sloop de nickname
        // :Alguno!me@ftd4linux.xs4all.nl
        tmp = strstr(nickname, "!");
        if (tmp) {
            len = tmp - nickname;
            for (i = 1; i < len; i++) {
                nickname[i - 1] = nickname[i];
            }
            nickname[len - 1] = '\0';
        }

        if (irc_data->on_mode_change_cb) {
            irc_data->on_mode_change_cb (irc_data, nickname, msg, mode);
        }
    }

// :Alguno!me@ftd4linux.2y.net PRIVMSG #ftd4linux :blaat
    memset(nickname, 0, sizeof(nickname));
    memset(msg, 0, sizeof(msg));
    ret = sscanf(regel, "%s PRIVMSG %s :%255[^\n]", &nickname, &channel_name, &msg);
    if (ret == 3) {
        char *tmp;
        int i, len;
        gboolean is_action = FALSE;
        gboolean is_privmsg;
        char *clean_msg;

        // Is dit bericht een pm bericht ?
        if (strcmp(channel_name, irc_data->channel) == 0) {
            is_privmsg = FALSE;
        } else {
            is_privmsg = TRUE;
        }

        // Sloop ACTION's
        if (msg[0] == '\001')
        {
            is_action = TRUE;
            for (i = 8; i < strlen(msg) - 1; i++)
                msg[i - 8] = msg[i];

            // Aan het einde van msg staat nog een niet-afdrukbaar teken, vervang deze gelijk door \0
            msg[strlen(msg) - 9] = '\0';
        }

        // Sloop de nickname
        // :Alguno!me@ftd4linux.xs4all.nl
        tmp = strstr(nickname, "!");
        if (tmp) {
            len = tmp - nickname;
            for (i = 1; i < len; i++) {
                nickname[i - 1] = nickname[i];
            }
            nickname[len - 1] = '\0';
        }

        // Ontdoe de message van allerlei exotische opmaak
        clean_msg = strip_color(msg);
        if (irc_data->on_msg_received_cb) {
            irc_data->on_msg_received_cb(irc_data, nickname, clean_msg, is_action, is_privmsg, irc_data->data);
        }
        g_free(clean_msg);
    }

    return TRUE;
}

void
ftd_gui_irc_send_msg(FtdGuiIRC *irc_data, const char *dest, const char *msg, gboolean is_action)
{
    char *command;

    g_assert(irc_data);
    g_assert(irc_data->io_channel);

    if (is_action) {
        command = g_strdup_printf("PRIVMSG %s :\001ACTION %s\001\n", dest, msg);
    } else {
        command = g_strdup_printf("PRIVMSG %s %s\n", dest, msg);
    }

    g_io_channel_write_chars(irc_data->io_channel, command, -1, NULL, NULL);
    g_io_channel_flush(irc_data->io_channel, NULL);

    g_free(command);
}

gboolean
ftd_gui_irc_connect(FtdGuiIRC *irc_data)
{
    int socket_fd;
    char *msg;

    struct addrinfo hints, *res, *ressave;
    int n;
    gboolean socket_success = FALSE;

    g_assert(irc_data);
    g_assert(irc_data->nickname);
    g_assert(irc_data->altNickname);
    g_assert(irc_data->channel);
    g_assert(irc_data->server);
    g_assert(irc_data->io_channel == NULL);

    irc_data->state = IRC_STATE_CONNECTING;
    if (irc_data->state_changed_cb) {
        irc_data->state_changed_cb(irc_data, IRC_STATE_DISCONNECTED, NULL, irc_data->data);
    }

    memset(&hints, 0, sizeof(struct addrinfo));

    hints.ai_family = AF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;

    n = getaddrinfo(irc_data->server, "6667", &hints, &res);

    /* Waarom moet windows nou weer alternatief doen ?? */
#ifdef WIN32
    if (n == WSAHOST_NOT_FOUND || n == WSATRY_AGAIN || n == WSANO_RECOVERY || n == WSANO_DATA) {
#else
    if (n < 0) {
#endif
        msg = g_strdup_printf("getaddrinfo() FAILED:\n%s", gai_strerror(n));

        irc_data->state = IRC_STATE_DISCONNECTED;
        if (irc_data->state_changed_cb) {
            irc_data->state_changed_cb(irc_data, IRC_STATE_CONNECTING, msg, irc_data->data);
        }

        g_free(msg);

        return FALSE;
    }

    ressave = res;

    socket_fd = 0;
    while (res) {
        socket_fd = socket(res->ai_family,
                           res->ai_socktype,
                           res->ai_protocol);

#ifdef WIN32
        if (!(socket_fd == INVALID_SOCKET)) {
#else
        if (!(socket_fd == -1)) {
#endif
            socket_success = TRUE;
            if (connect(socket_fd, res->ai_addr, (int) res->ai_addrlen) == 0)
                break;

#ifdef WIN32
            closesocket(socket_fd);
#else
            close(socket_fd);
#endif
            socket_fd = 0;
        }

        res=res->ai_next;
    }

    if (socket_fd == 0) {
        freeaddrinfo(ressave);

        if (socket_success) {
            msg = g_strdup_printf("connect() FAILED:\n%s", strerror(errno));
        } else {
            msg = g_strdup_printf("socket() FAILED:\n%s", strerror(errno));
        }

        irc_data->state = IRC_STATE_DISCONNECTED;
        if (irc_data->state_changed_cb) {
            irc_data->state_changed_cb(irc_data, IRC_STATE_CONNECTING, msg, irc_data->data);
        }

        g_free(msg);

        return FALSE;
    }

    freeaddrinfo(ressave);

#ifdef DEBUG
    g_print("socket_fd = %i\n", socket_fd);
#endif

    irc_data->io_channel = g_io_channel_unix_new(socket_fd);
    g_io_add_watch(irc_data->io_channel, G_IO_IN /*| G_IO_ERR | G_IO_HUP*/, parse_irc_message, irc_data);

    msg = g_strdup_printf("NICK %s\nUSER ftd4linux ftd4linux ftd4linux :FTD4Linux\r\n", irc_data->nickname);
    g_io_channel_write_chars(irc_data->io_channel, msg, -1, NULL, NULL);
    g_io_channel_flush(irc_data->io_channel, NULL);
    g_free(msg);

    return TRUE;
}

void
ftd_gui_irc_disconnect(FtdGuiIRC *irc_data)
{
    FtdGuiIRCState old_state;

    g_assert(irc_data);
    g_assert(irc_data->io_channel);

    g_io_channel_shutdown(irc_data->io_channel, TRUE, NULL);
    g_io_channel_unref(irc_data->io_channel);
    irc_data->io_channel = NULL;

    old_state = irc_data->state;
    irc_data->state = IRC_STATE_DISCONNECTED;

    if (irc_data->state_changed_cb) {
        irc_data->state_changed_cb(irc_data, old_state, NULL, irc_data->data);
    }
}
