/* ====================================================================
 * FTD4Linux Software License, Version 1.0
 *
 * Copyright (c) 2000-2004 The FTD4Linux Development Team.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        FTD4Linux Development Team (http://www.ftd4linux.2y.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "FTD" and "FTD4Linux" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact alguno@ftd4linux.2y.net.
 *
 * 5. Products derived from this software may not be called "FTD4Linux",
 *    nor may "FTD4Linux" appear in their name, without prior written
 *    permission of the FTD4Linux Development Team.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE FTD4LINUX DEVELOPMENT TEAM OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include "ftd_gui_main.h"
#include "ftd_gui_message.h"
#include "ftd_gui_mail.h"
#include "ftd_gui_center_eventlist.h"
#include "ftd_gui_errhandler.h"

#ifdef HAVE_GTKSPELL
#include <gtkspell/gtkspell.h>
#endif

FtdGuiMessage*
ftd_gui_message_new (const char *geadresseerde, const char *subject, const char *contents, int ftd_postid, FtdEventForumReactie *forum_reactie)
{
    FtdGuiMessage *message = (FtdGuiMessage *) g_malloc(sizeof(FtdGuiMessage));
    FtdGuiMessageData *message_data = (FtdGuiMessageData *) g_malloc(sizeof(FtdGuiMessageData));

    GtkWidget *vbox1;
    GtkWidget *hbox2;
    GtkWidget *label2;
    GtkWidget *label3;
    GtkWidget *scrolledwindow2;
    GtkWidget *viewport1;
    GtkWidget *vpaned1;
    GtkWidget *vpaned2;
    GtkWidget *scrolledwindow5;
    GtkWidget *hbox1;
    GtkWidget *table1;
    GtkWidget *cmdBold;
    GtkWidget *cmdItalic;
    GtkWidget *cmdUnderline;
    GtkWidget *cmdQuote;
#if 0
    GtkWidget *cmdFTD;
#endif
    GtkWidget *table2;
    GtkWidget *vbox2;
//    GtkWidget *chkVoorbeeld;
    GtkWidget *cmdCancel;
    GError *err = NULL;

    message_data->message = message;

    message->postid = ftd_postid;

    message->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (message->window), "Voeg een bericht toe");
    gtk_window_set_position (GTK_WINDOW (message->window), GTK_WIN_POS_CENTER_ALWAYS);
    gtk_window_set_modal(GTK_WINDOW (message->window), TRUE);
    gtk_widget_set_size_request (message->window, 580, 400);
    if (ftd_postid == 0 && gui->mail) {
        gtk_window_set_transient_for(GTK_WINDOW(message->window), GTK_WINDOW(gui->mail->window));
    } else {
        gtk_window_set_transient_for(GTK_WINDOW(message->window), GTK_WINDOW(gui->window));
    }

    vbox1 = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox1);
    gtk_container_add (GTK_CONTAINER (message->window), vbox1);

    hbox2 = gtk_hbox_new (FALSE, 0);
    if (ftd_postid == 0) {
        gtk_widget_show (hbox2);
    }
    gtk_box_pack_start (GTK_BOX (vbox1), hbox2, FALSE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox2), 2);

    label2 = gtk_label_new ("Aan :");
    gtk_widget_show (label2);
    gtk_box_pack_start (GTK_BOX (hbox2), label2, FALSE, FALSE, 0);
    gtk_misc_set_padding (GTK_MISC (label2), 4, 10);

    message->txtAan = gtk_entry_new ();
    gtk_widget_show (message->txtAan);
    gtk_box_pack_start (GTK_BOX (hbox2), message->txtAan, TRUE, TRUE, 0);
    if (geadresseerde) {
        if (strlen(geadresseerde) > 0) {
            gtk_entry_set_text(GTK_ENTRY(message->txtAan), geadresseerde);
            gtk_widget_set_sensitive(message->txtAan, FALSE);
        }
    }

    label3 = gtk_label_new ("Onderwerp :");
    gtk_widget_show (label3);
    gtk_box_pack_start (GTK_BOX (hbox2), label3, FALSE, FALSE, 0);
    gtk_misc_set_padding (GTK_MISC (label3), 4, 10);

    message->txtSubject = gtk_entry_new ();
    gtk_widget_show (message->txtSubject);
    gtk_box_pack_start (GTK_BOX (hbox2), message->txtSubject, TRUE, TRUE, 0);
    if (subject) {
        if (strlen(subject) > 0) {
            gtk_entry_set_text(GTK_ENTRY(message->txtSubject), subject);
        }
    }

    scrolledwindow2 = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_show (scrolledwindow2);
    gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow2, TRUE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (scrolledwindow2), 2);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow2), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

    viewport1 = gtk_viewport_new (NULL, NULL);
    gtk_widget_show (viewport1);
    gtk_container_add (GTK_CONTAINER (scrolledwindow2), viewport1);

    vpaned1 = gtk_vpaned_new ();
    gtk_widget_show (vpaned1);
    gtk_container_add (GTK_CONTAINER (viewport1), vpaned1);

    gtk_paned_set_position (GTK_PANED (vpaned1), 130);

/*
    scrolledwindow4 = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_show (scrolledwindow4);
    gtk_paned_pack1 (GTK_PANED (vpaned1), scrolledwindow4, FALSE, TRUE);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow4), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

    message->txtQuote = gtk_text_view_new ();
    gtk_widget_show (message->txtQuote);
    gtk_container_add (GTK_CONTAINER (scrolledwindow4), message->txtQuote);
*/

    vpaned2 = gtk_vpaned_new ();
    gtk_widget_show (vpaned2);
    gtk_paned_pack2 (GTK_PANED (vpaned1), vpaned2, TRUE, TRUE);

    scrolledwindow5 = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_show (scrolledwindow5);
    gtk_paned_pack1 (GTK_PANED (vpaned2), scrolledwindow5, FALSE, TRUE);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow5), GTK_POLICY_AUTOMATIC, GTK_POLICY_NEVER);

    message->txtBericht = gtk_text_view_new ();
#ifdef HAVE_GTKSPELL
    if (!gtkspell_new_attach(GTK_TEXT_VIEW(message->txtBericht), "nl", &err)) {
        char *msg = g_strdup_printf("Initialiseren van de spellingscontrole mislukt:\n%s\nSpellingscontrole wordt uitgeschakeld", err->message);
        ftd_gui_dialog_show(NULL, msg, gui->window, FALSE);
        g_error_free(err);
        g_free(msg);
    }
#endif
    gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(message->txtBericht), GTK_WRAP_WORD);
    gtk_widget_show (message->txtBericht);
    gtk_container_add (GTK_CONTAINER (scrolledwindow5), message->txtBericht);

    if (contents && strlen(contents) > 0) {
        FtdString *src = ftd_string_new(contents);
        FtdString *new = ftd_string_replace(src, "<br />", "");
        GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(message->txtBericht));
        gtk_text_buffer_set_text(buf, ftd_string_value(new), -1);
        ftd_string_free(new);
        ftd_string_free(src);
    }

    if (forum_reactie) {
        GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(message->txtBericht));
        FtdString *new = ftd_string_replace(forum_reactie->body, "<br />", "\n");
        char *msg = g_strdup_printf("[quote=%s]%s[/quote]", ftd_string_value(forum_reactie->author), ftd_string_value(new));
        gtk_text_buffer_set_text(buf, msg, -1);
        ftd_string_free(new);
        g_free(msg);
    }

    if (geadresseerde && subject) {
        if (strlen(geadresseerde) > 0 && strlen(subject) > 0) {
            gtk_widget_grab_focus(message->txtBericht);
        }
    }

#if 0
#ifndef HAVE_LIBGTKHTML
    message->preview = gtk_moz_embed_new();
    //gtk_moz_embed_load_url(GTK_MOZ_EMBED(message->preview), PACKAGE_DATA_DIR "/" PACKAGE "/html/selecteer_mail.html");
    gtk_paned_pack2 (GTK_PANED (vpaned2), message->preview, TRUE, TRUE);
    gtk_widget_show (message->preview);
#endif
#endif

    hbox1 = gtk_hbox_new (FALSE, 0);
    gtk_widget_show (hbox1);
    gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox1), 3);

    table1 = gtk_table_new (2, 2, FALSE);
    gtk_widget_show (table1);
    gtk_box_pack_start (GTK_BOX (hbox1), table1, FALSE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (table1), 2);

    cmdBold = gtk_button_new_with_mnemonic ("B");
    gtk_widget_show (cmdBold);
    gtk_table_attach (GTK_TABLE (table1), cmdBold, 0, 1, 0, 1,
                      (GtkAttachOptions) (GTK_FILL),
                      (GtkAttachOptions) (0), 0, 0);
    gtk_widget_set_size_request (cmdBold, 45, 28);

    cmdItalic = gtk_button_new_with_mnemonic ("i");
    gtk_widget_show (cmdItalic);
    gtk_table_attach (GTK_TABLE (table1), cmdItalic, 1, 2, 0, 1,
                      (GtkAttachOptions) (GTK_FILL),
                      (GtkAttachOptions) (0), 0, 0);
    gtk_widget_set_size_request (cmdItalic, 45, 28);

    cmdUnderline = gtk_button_new_with_mnemonic ("U");
    gtk_widget_show (cmdUnderline);
    gtk_table_attach (GTK_TABLE (table1), cmdUnderline, 0, 1, 1, 2,
                      (GtkAttachOptions) (GTK_FILL),
                      (GtkAttachOptions) (0), 0, 0);
    gtk_widget_set_size_request (cmdUnderline, 45, 28);

    cmdQuote = gtk_button_new_with_mnemonic ("[..]");
    gtk_widget_show (cmdQuote);
    gtk_table_attach (GTK_TABLE (table1), cmdQuote, 1, 2, 1, 2,
                      (GtkAttachOptions) (GTK_FILL),
                      (GtkAttachOptions) (0), 0, 0);
    gtk_widget_set_size_request (cmdQuote, 45, 28);

#if 0
    cmdFTD = gtk_button_new_with_mnemonic ("[ftd]...[/ftd]");
    gtk_widget_show (cmdFTD);
    gtk_box_pack_start (GTK_BOX (hbox1), cmdFTD, FALSE, FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (cmdFTD), 2);
#endif

    table2 = gtk_table_new (2, 7, FALSE);
    gtk_widget_show (table2);
    gtk_box_pack_start (GTK_BOX (hbox1), table2, FALSE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (table2), 2);

    ftd_gui_message_verwerk_smilies (table2, message);

    vbox2 = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox2);
    gtk_box_pack_start (GTK_BOX (hbox1), vbox2, FALSE, TRUE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (vbox2), 2);

    //chkVoorbeeld = gtk_check_button_new_with_mnemonic ("Voorbeeld");
    //gtk_widget_show (chkVoorbeeld);
    //gtk_box_pack_start (GTK_BOX (vbox2), chkVoorbeeld, FALSE, FALSE, 0);

    message->btnVerzend = gtk_button_new_with_mnemonic ("OK");
    gtk_widget_show (message->btnVerzend);
    gtk_box_pack_start (GTK_BOX (vbox2), message->btnVerzend, FALSE, FALSE, 0);
    gtk_widget_set_size_request (message->btnVerzend, -1, 28);

    cmdCancel = gtk_button_new_with_mnemonic("Annuleren");
    gtk_widget_show(cmdCancel);
    gtk_box_pack_start (GTK_BOX (vbox2), cmdCancel, FALSE, FALSE, 0);
    gtk_widget_set_size_request (cmdCancel, -1, 28);

    message->statusbar = gtk_statusbar_new ();
    gtk_widget_show (message->statusbar);
    gtk_box_pack_start (GTK_BOX (vbox1), message->statusbar, FALSE, FALSE, 0);

    /* Signal handlers */
    g_signal_connect (G_OBJECT (cmdBold), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdBold), message);
    g_signal_connect (G_OBJECT (cmdItalic), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdItalic), message);
    g_signal_connect (G_OBJECT (cmdUnderline), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdUnderline), message);
    g_signal_connect (G_OBJECT (cmdQuote), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdQuote), message);
#if 0
    g_signal_connect (G_OBJECT (cmdFTD), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdFTD), message);
#endif

    //g_signal_connect (G_OBJECT (chkVoorbeeld), "toggled",
    //                  G_CALLBACK (ftd_gui_message_callback_chkVoorbeeld), message);

    g_signal_connect (G_OBJECT (message->btnVerzend), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdVerzend), message_data);
    g_signal_connect (G_OBJECT (cmdCancel), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdCancel), message->window);

    /* Voor het mozilla componentje hebben we een speciale behandeling
     * deze moet namelijk eerst getoond worden met gtk_widget_show omdat
     * deze anders het programma laat crashen als dit venster gesloten wordt
     * Omdat we standaard het mozilla componentje niet hoeven te tonen (alleen
     * als de optie 'Voorbeeld' is aangevinkt) laten we na het realize
     * signaal dit componentje onzichtbaar maken */
#ifndef HAVE_LIBGTKHTML
#if 0
    g_signal_connect_after (GTK_WIDGET (message->preview), "realize", G_CALLBACK(ftd_gui_message_preview_realized), NULL);
#endif
#endif

    gtk_widget_show (message->window);

    return message;
}

void
ftd_gui_message_preview_realized (GtkWidget *caller, gpointer data)
{
    gtk_widget_hide(caller);
}

void
ftd_gui_message_verwerk_smilies(GtkWidget *table2, FtdGuiMessage *message)
{
    GtkWidget *cmdSmileBiggrin;
    GtkWidget *cmdSmileShiny;
    GtkWidget *cmdSmileConfused;
    GtkWidget *cmdSmileClown;
    GtkWidget *cmdSmileFrusty;
    GtkWidget *cmdSmileRedface;
    GtkWidget *cmdSmileSleephappy;
    GtkWidget *cmdSmileDevil;
    GtkWidget *cmdSmileWink;
    GtkWidget *cmdSmileCry;
    GtkWidget *cmdSmilePuh2;
    GtkWidget *cmdSmilePukey;
    GtkWidget *cmdSmileCool;
    GtkWidget *cmdSmileFrown;
    GtkWidget *image;
        char *volledigeFilename;

    cmdSmileBiggrin = gtk_button_new();
    gtk_widget_show (cmdSmileBiggrin);
    gtk_table_attach (GTK_TABLE (table2), cmdSmileBiggrin, 0, 1, 0, 1,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);

    volledigeFilename = ftd_file_parse_data_dir("/emoticons/biggrin.gif");
    image = gtk_image_new_from_file (volledigeFilename);
    g_free(volledigeFilename);

    gtk_widget_show (image);
    gtk_container_add (GTK_CONTAINER (cmdSmileBiggrin), image);
    gtk_widget_set_size_request (cmdSmileBiggrin, 45, 28);

    cmdSmileShiny = gtk_button_new();
    gtk_widget_show (cmdSmileShiny);
    gtk_table_attach (GTK_TABLE (table2), cmdSmileShiny, 1, 2, 0, 1,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);

    volledigeFilename = ftd_file_parse_data_dir("/emoticons/shiny.gif");
    image = gtk_image_new_from_file (volledigeFilename);
    g_free(volledigeFilename);

    gtk_widget_show (image);
    gtk_container_add (GTK_CONTAINER (cmdSmileShiny), image);
    gtk_widget_set_size_request (cmdSmileShiny, 45, 28);

    cmdSmileConfused = gtk_button_new();
    gtk_widget_show (cmdSmileConfused);
    gtk_table_attach (GTK_TABLE (table2), cmdSmileConfused, 2, 3, 0, 1,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);

    volledigeFilename = ftd_file_parse_data_dir("/emoticons/confused.gif");
    image = gtk_image_new_from_file (volledigeFilename);
    g_free(volledigeFilename);

    gtk_widget_show (image);
    gtk_container_add (GTK_CONTAINER (cmdSmileConfused), image);
    gtk_widget_set_size_request (cmdSmileConfused, 45, 28);

    cmdSmileClown = gtk_button_new();
    gtk_widget_show (cmdSmileClown);
    gtk_table_attach (GTK_TABLE (table2), cmdSmileClown, 3, 4, 0, 1,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);

    volledigeFilename = ftd_file_parse_data_dir("/emoticons/clown.gif");
    image = gtk_image_new_from_file (volledigeFilename);
    g_free(volledigeFilename);

    gtk_widget_show (image);
    gtk_container_add (GTK_CONTAINER (cmdSmileClown), image);
    gtk_widget_set_size_request (cmdSmileClown, 45, 28);

    cmdSmileFrusty = gtk_button_new();
    gtk_widget_show (cmdSmileFrusty);
    gtk_table_attach (GTK_TABLE (table2), cmdSmileFrusty, 4, 5, 0, 1,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);

    volledigeFilename = ftd_file_parse_data_dir("/emoticons/frusty.gif");
    image = gtk_image_new_from_file (volledigeFilename);
    g_free(volledigeFilename);

    gtk_widget_show (image);
    gtk_container_add (GTK_CONTAINER (cmdSmileFrusty), image);
    gtk_widget_set_size_request (cmdSmileFrusty, 45, 28);

    cmdSmileRedface = gtk_button_new();
    gtk_widget_show (cmdSmileRedface);
    gtk_table_attach (GTK_TABLE (table2), cmdSmileRedface, 5, 6, 0, 1,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);

    volledigeFilename = ftd_file_parse_data_dir("/emoticons/redface.gif");
    image = gtk_image_new_from_file (volledigeFilename);
    g_free(volledigeFilename);

    gtk_widget_show (image);
    gtk_container_add (GTK_CONTAINER (cmdSmileRedface), image);
    gtk_widget_set_size_request (cmdSmileRedface, 45, 28);

    cmdSmileSleephappy = gtk_button_new();
    gtk_widget_show (cmdSmileSleephappy);
    gtk_table_attach (GTK_TABLE (table2), cmdSmileSleephappy, 6, 7, 0, 1,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);

    volledigeFilename = ftd_file_parse_data_dir("/emoticons/sleephappy.gif");
    image = gtk_image_new_from_file (volledigeFilename);
    g_free(volledigeFilename);

    gtk_widget_show (image);
    gtk_container_add (GTK_CONTAINER (cmdSmileSleephappy), image);
    gtk_widget_set_size_request (cmdSmileSleephappy, 45, 28);

    cmdSmileDevil = gtk_button_new();
    gtk_widget_show (cmdSmileDevil);
    gtk_table_attach (GTK_TABLE (table2), cmdSmileDevil, 0, 1, 1, 2,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);

    volledigeFilename = ftd_file_parse_data_dir("/emoticons/devil.gif");
    image = gtk_image_new_from_file (volledigeFilename);
    g_free(volledigeFilename);

    gtk_widget_show (image);
    gtk_container_add (GTK_CONTAINER (cmdSmileDevil), image);
    gtk_widget_set_size_request (cmdSmileDevil, 45, 28);

    cmdSmileWink = gtk_button_new();
    gtk_widget_show (cmdSmileWink);
    gtk_table_attach (GTK_TABLE (table2), cmdSmileWink, 1, 2, 1, 2,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);

    volledigeFilename = ftd_file_parse_data_dir("/emoticons/wink.gif");
    image = gtk_image_new_from_file (volledigeFilename);
    g_free(volledigeFilename);

    gtk_widget_show (image);
    gtk_container_add (GTK_CONTAINER (cmdSmileWink), image);
    gtk_widget_set_size_request (cmdSmileWink, 45, 28);

    cmdSmileCry = gtk_button_new();
    gtk_widget_show (cmdSmileCry);
    gtk_table_attach (GTK_TABLE (table2), cmdSmileCry, 2, 3, 1, 2,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);

    volledigeFilename = ftd_file_parse_data_dir("/emoticons/cry.gif");
    image = gtk_image_new_from_file (volledigeFilename);
    g_free(volledigeFilename);

    gtk_widget_show (image);
    gtk_container_add (GTK_CONTAINER (cmdSmileCry), image);
    gtk_widget_set_size_request (cmdSmileCry, 45, 28);

    cmdSmilePuh2 = gtk_button_new();
    gtk_widget_show (cmdSmilePuh2);
    gtk_table_attach (GTK_TABLE (table2), cmdSmilePuh2, 3, 4, 1, 2,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);

    volledigeFilename = ftd_file_parse_data_dir("/emoticons/puh2.gif");
    image = gtk_image_new_from_file (volledigeFilename);
    g_free(volledigeFilename);

    gtk_widget_show (image);
    gtk_container_add (GTK_CONTAINER (cmdSmilePuh2), image);
    gtk_widget_set_size_request (cmdSmilePuh2, 45, 28);

    cmdSmilePukey = gtk_button_new();
    gtk_widget_show (cmdSmilePukey);
    gtk_table_attach (GTK_TABLE (table2), cmdSmilePukey, 4, 5, 1, 2,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);

    volledigeFilename = ftd_file_parse_data_dir("/emoticons/pukey.gif");
    image = gtk_image_new_from_file (volledigeFilename);
    g_free(volledigeFilename);

    gtk_widget_show (image);
    gtk_container_add (GTK_CONTAINER (cmdSmilePukey), image);
    gtk_widget_set_size_request (cmdSmilePukey, 45, 28);

    cmdSmileCool = gtk_button_new();
    gtk_widget_show (cmdSmileCool);
    gtk_table_attach (GTK_TABLE (table2), cmdSmileCool, 5, 6, 1, 2,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);

    volledigeFilename = ftd_file_parse_data_dir("/emoticons/cool.gif");
    image = gtk_image_new_from_file (volledigeFilename);
    g_free(volledigeFilename);

    gtk_widget_show (image);
    gtk_container_add (GTK_CONTAINER (cmdSmileCool), image);
    gtk_widget_set_size_request (cmdSmileCool, 45, 28);

    cmdSmileFrown = gtk_button_new();
    gtk_widget_show (cmdSmileFrown);
    gtk_table_attach (GTK_TABLE (table2), cmdSmileFrown, 6, 7, 1, 2,
                      (GtkAttachOptions) (0),
                      (GtkAttachOptions) (0), 0, 0);

    volledigeFilename = ftd_file_parse_data_dir("/emoticons/frown.gif");
    image = gtk_image_new_from_file (volledigeFilename);
    g_free(volledigeFilename);

    gtk_widget_show (image);
    gtk_container_add (GTK_CONTAINER (cmdSmileFrown), image);
    gtk_widget_set_size_request (cmdSmileFrown, 45, 28);

    /* Signal handlers */
    g_signal_connect (G_OBJECT (cmdSmileBiggrin), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdSmileBiggrin), message);
    g_signal_connect (G_OBJECT (cmdSmileShiny), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdSmileShiny), message);
    g_signal_connect (G_OBJECT (cmdSmileConfused), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdSmileConfused), message);
    g_signal_connect (G_OBJECT (cmdSmileClown), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdSmileClown), message);
    g_signal_connect (G_OBJECT (cmdSmileFrusty), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdSmileFrusty), message);
    g_signal_connect (G_OBJECT (cmdSmileRedface), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdSmileRedface), message);
    g_signal_connect (G_OBJECT (cmdSmileSleephappy), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdSmileSleephappy), message);
    g_signal_connect (G_OBJECT (cmdSmileDevil), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdSmileDevil), message);
    g_signal_connect (G_OBJECT (cmdSmileWink), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdSmileWink), message);
    g_signal_connect (G_OBJECT (cmdSmileCry), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdSmileCry), message);
    g_signal_connect (G_OBJECT (cmdSmilePuh2), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdSmilePuh2), message);
    g_signal_connect (G_OBJECT (cmdSmilePukey), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdSmilePukey), message);
    g_signal_connect (G_OBJECT (cmdSmileCool), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdSmileCool), message);
    g_signal_connect (G_OBJECT (cmdSmileFrown), "clicked",
                      G_CALLBACK (ftd_gui_message_callback_cmdSmileFrown), message);
}

void
ftd_gui_message_callback_chkVoorbeeld(GtkWidget *caller, gpointer data)
{

}

void
ftd_gui_message_insert_text (FtdGuiMessage *message, char *beginTag, char *endTag)
{
    GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (message->txtBericht));

    if (endTag == NULL) {
        gtk_text_buffer_insert_at_cursor (buffer, beginTag, (gint) strlen(beginTag));
    } else {
//        GtkTextIter *beginPos, *eindPos;
        gboolean retval = TRUE; // = gtk_text_buffer_get_selection_bounds (buffer, beginPos, eindPos);

        if (retval != TRUE) { /* Hebben we wel een selectie ? */
            /* Geen selectie, return */
            return;
        }

/* FIXME: Iemand een idee hoe we hier tekst kunnen toevoegen aan het begin en einde van een selectie ?
        gtk_text_buffer_insert_text (buffer, beginTag, strlen(beginTag), beginPos);
        eindPos += strlen(beginTag);
        gtk_text_buffer_insert_text (buffer, endTag, strlen(endTag), eindPos);
*/
    }
}

gboolean
ftd_gui_message_callback_cmdBold(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, "[b]", "[/b]");

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdItalic(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, "[i]", "[/i]");

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdUnderline(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, "[u]", "[/u]");

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdQuote(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, "[quote]", "[/quote]");

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdFTD(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, "[ftd]", "[/ftd]");

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdSmileBiggrin(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, " :D ", NULL);
    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdSmileShiny(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, " *D ", NULL);

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdSmileConfused(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, " :? ", NULL);

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdSmileClown(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, " :+ ", NULL);

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdSmileFrusty(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, " |:( ", NULL);

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdSmileRedface(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, " :o ", NULL);

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdSmileSleephappy(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, " :Z ", NULL);

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdSmileDevil(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, " }:) ", NULL);

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdSmileWink(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, " ;) ", NULL);

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdSmileCry(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, " :'( ", NULL);

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdSmilePuh2(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, " :P ", NULL);

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdSmilePukey(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, " :r ", NULL);

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdSmileCool(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, " 8) ", NULL);

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdSmileFrown(GtkWidget *caller, gpointer data)
{
    FtdGuiMessage *message = (FtdGuiMessage *)data;
    g_assert(message);

    ftd_gui_message_insert_text (message, " :( ", NULL);

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdVerzend(GtkWidget *caller, gpointer data)
{
    FtdGuiMessageData *message_data = (FtdGuiMessageData *) data;
    GtkTextIter start, end;
    GtkTextBuffer *buffer;
    gchar *tekst;
    FtdErrorCode retval;

    g_assert(message_data);

    buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(message_data->message->txtBericht));
    gtk_text_buffer_get_start_iter(buffer, &start);
    gtk_text_buffer_get_end_iter(buffer, &end);
    tekst = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);

    if (message_data->message->postid == 0) {
        if (!strcmp(gtk_entry_get_text(GTK_ENTRY(message_data->message->txtAan)), "")) {
            ftd_gui_dialog_show(NULL, "Er is geen geadresseerde ingevuld.", message_data->message->window, FALSE);
            return FALSE;
        }

        if (!strcmp(gtk_entry_get_text(GTK_ENTRY(message_data->message->txtSubject)), "")) {
            ftd_gui_dialog_show(NULL, "Er is geen onderwerp ingevuld.", message_data->message->window, FALSE);
            return FALSE;
        }
    }

    if (!strcmp(tekst, "")) {
        ftd_gui_dialog_show(NULL, "Er is geen bericht opgegeven dat verzonden moet worden", message_data->message->window, FALSE);
        return FALSE;
    }

    gtk_window_set_modal(GTK_WINDOW(message_data->message->window), FALSE);
    gtk_widget_set_sensitive(message_data->message->window, FALSE);

    if (message_data->message->postid == 0) {
        char *date;
        time_t stamp;
        char tmp[1024];
        char *errmsg;

        // We hebben een FTD Mail bericht
        message_data->bericht = (FtdMailBericht *) g_malloc(sizeof(FtdMailBericht));
        // FIXME: tekst kan UTF8 code bevatten
        message_data->bericht->from = ftd_string_new(gtk_entry_get_text(GTK_ENTRY(message_data->message->txtAan)));
        message_data->bericht->subject = ftd_string_new(gtk_entry_get_text(GTK_ENTRY(message_data->message->txtSubject)));
        message_data->bericht->body = ftd_string_new(tekst);

        retval = ftd_mail_post(message_data->bericht, &errmsg);

        stamp = time(NULL);
        date = ftd_string_date(stamp);

        if (retval > 0 || retval == -1) {
            if (retval > 0) {
                ftd_gui_errhandler(gui->mail->window, FTD_ONDERDEEL_POSTMAIL, retval, errmsg);
            } else {
                ftd_gui_dialog_show(NULL, "Versturen van FTD Mail bericht mislukt", message_data->message->window, TRUE);
            }
            snprintf(tmp, sizeof(tmp), "%s: Versturen van FTD Mail bericht mislukt!", date);
            g_free(date);
            if (gui->mail) {
                gtk_statusbar_push(GTK_STATUSBAR(gui->mail->statusbar), 0, tmp);
            }
            gtk_widget_set_sensitive(message_data->message->window, TRUE);
            gtk_window_set_modal(GTK_WINDOW(message_data->message->window), TRUE);
            return FALSE;
        }

        gtk_widget_destroy(message_data->message->window);
        g_free(message_data);
        snprintf(tmp, sizeof(tmp), "%s: Versturen van FTD Mail bericht geslaagd - %i nieuw(e) bericht(en); %i berichten totaal", date, -retval, ftd_mail_get_num_messages());
        g_free(date);
        if (gui->mail) {
            gtk_statusbar_push(GTK_STATUSBAR(gui->mail->statusbar), 0, tmp);
            ftd_gui_mail_treeVerzonden_update(NULL);
        }
    } else {
        char tmp[64];
        char *errmsg;

        // We hebben een FTD post reactie
        message_data->bericht = NULL;

        retval = ftd_events_forum_post(tekst, message_data->message->postid, message_data, &errmsg);

        if (retval != ERROR_NONE) {
            ftd_gui_errhandler(gui->window, FTD_ONDERDEEL_EVENTINFO_FORUM, retval, errmsg);
            gtk_widget_set_sensitive(message_data->message->window, TRUE);
            return FALSE;
        }

        /* De event info moet nu opnieuw opgehaald en getoont worden */
        snprintf(tmp, sizeof(tmp), "iets.html#post%i", message_data->message->postid);
        ftd_gui_center_eventlist_open_uri(NULL, tmp, NULL);

        gtk_widget_destroy(message_data->message->window);
        g_free(message_data);
    }

    g_free(tekst);

    return FALSE;
}

gboolean
ftd_gui_message_callback_cmdCancel(GtkWidget *caller, gpointer data)
{
    gtk_widget_destroy((GtkWidget *) data);
    return FALSE;
}
