/* ====================================================================
 * FTD4Linux Software License, Version 1.0
 *
 * Copyright (c) 2000-2004 The FTD4Linux Development Team.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        FTD4Linux Development Team (http://www.ftd4linux.2y.net/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "FTD" and "FTD4Linux" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact alguno@ftd4linux.2y.net.
 *
 * 5. Products derived from this software may not be called "FTD4Linux",
 *    nor may "FTD4Linux" appear in their name, without prior written
 *    permission of the FTD4Linux Development Team.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE FTD4LINUX DEVELOPMENT TEAM OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include "ftd_gui_main.h"
#include "ftd_gui_pixbuf.h"
#include "ftd_gui_toolbar.h"

void
ftd_gui_toolbar_new(void)
{
    FtdGuiToolbar *toolbar;
    GtkWidget *separator1;
    GtkWidget *separator2;
    GtkWidget *vbox;
    GtkWidget *label;
    GdkPixbuf *pixbuf;
    GtkRequisition req;

    g_assert(gui);

    toolbar = (FtdGuiToolbar *) g_malloc(sizeof(FtdGuiToolbar));
    gui->toolbar = toolbar;
    toolbar->widget = gtk_toolbar_new ();
    gtk_toolbar_set_style (GTK_TOOLBAR (toolbar->widget), GTK_TOOLBAR_ICONS);

    toolbar->btnUpdaten = gtk_button_new();
    vbox = gtk_vbox_new(FALSE, 0);
    label = gtk_label_new("Updaten");
    gtk_button_set_relief(GTK_BUTTON(toolbar->btnUpdaten), GTK_RELIEF_NONE);
#if GTK_CHECK_VERSION(2,4,0)
    gtk_button_set_focus_on_click(GTK_BUTTON(toolbar->btnUpdaten), FALSE);
#endif
    pixbuf = ftd_gui_main_get_pixbuf("icon-updaten_off");
    toolbar->imgUpdaten = gtk_image_new_from_pixbuf(pixbuf);
    gtk_box_pack_start(GTK_BOX(vbox), toolbar->imgUpdaten, TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
    gtk_container_add(GTK_CONTAINER(toolbar->btnUpdaten), vbox);
    gtk_container_add(GTK_CONTAINER(toolbar->widget), toolbar->btnUpdaten);

    toolbar->btnConfig = gtk_button_new();
    vbox = gtk_vbox_new(FALSE, 0);
    label = gtk_label_new("Config");
    gtk_button_set_relief(GTK_BUTTON(toolbar->btnConfig), GTK_RELIEF_NONE);
#if GTK_CHECK_VERSION(2,4,0)
    gtk_button_set_focus_on_click(GTK_BUTTON(toolbar->btnConfig), FALSE);
#endif
    pixbuf = ftd_gui_main_get_pixbuf("icon-config_off");
    toolbar->imgConfig = gtk_image_new_from_pixbuf(pixbuf);
    gtk_box_pack_start(GTK_BOX(vbox), toolbar->imgConfig, TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
    gtk_container_add(GTK_CONTAINER(toolbar->btnConfig), vbox);
    gtk_container_add(GTK_CONTAINER(toolbar->widget), toolbar->btnConfig);

    toolbar->btnToevoegen = gtk_button_new();
    vbox = gtk_vbox_new(FALSE, 0);
    label = gtk_label_new("Toevoegen");
    gtk_button_set_relief(GTK_BUTTON(toolbar->btnToevoegen), GTK_RELIEF_NONE);
#if GTK_CHECK_VERSION(2,4,0)
    gtk_button_set_focus_on_click(GTK_BUTTON(toolbar->btnToevoegen), FALSE);
#endif
    pixbuf = ftd_gui_main_get_pixbuf("icon-toevoegen_off");
    toolbar->imgToevoegen = gtk_image_new_from_pixbuf(pixbuf);
    gtk_box_pack_start(GTK_BOX(vbox), toolbar->imgToevoegen, TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
    gtk_container_add(GTK_CONTAINER(toolbar->btnToevoegen), vbox);
    gtk_container_add(GTK_CONTAINER(toolbar->widget), toolbar->btnToevoegen);

    toolbar->btnMail = gtk_button_new();
    vbox = gtk_vbox_new(FALSE, 0);
    label = gtk_label_new("Mail");
    gtk_button_set_relief(GTK_BUTTON(toolbar->btnMail), GTK_RELIEF_NONE);
#if GTK_CHECK_VERSION(2,4,0)
    gtk_button_set_focus_on_click(GTK_BUTTON(toolbar->btnMail), FALSE);
#endif
    pixbuf = ftd_gui_main_get_pixbuf("icon-mail_off");
    toolbar->imgMail = gtk_image_new_from_pixbuf(pixbuf);
    gtk_box_pack_start(GTK_BOX(vbox), toolbar->imgMail, TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
    gtk_container_add(GTK_CONTAINER(toolbar->btnMail), vbox);
    gtk_container_add(GTK_CONTAINER(toolbar->widget), toolbar->btnMail);

    separator1 = gtk_vseparator_new();
    gtk_widget_set_size_request(separator1, 15 , 65);
    gtk_container_add(GTK_CONTAINER(toolbar->widget), separator1);

    toolbar->btnNieuw = gtk_button_new();
    vbox = gtk_vbox_new(FALSE, 0);
    label = gtk_label_new("Nieuw");
    gtk_button_set_relief(GTK_BUTTON(toolbar->btnNieuw), GTK_RELIEF_NONE);
#if GTK_CHECK_VERSION(2,4,0)
    gtk_button_set_focus_on_click(GTK_BUTTON(toolbar->btnNieuw), FALSE);
#endif
    pixbuf = ftd_gui_main_get_pixbuf("icon-nieuw_off");
    toolbar->imgNieuw = gtk_image_new_from_pixbuf(pixbuf);
    gtk_box_pack_start(GTK_BOX(vbox), toolbar->imgNieuw, TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
    gtk_container_add(GTK_CONTAINER(toolbar->btnNieuw), vbox);
    gtk_container_add(GTK_CONTAINER(toolbar->widget), toolbar->btnNieuw);
    gtk_widget_set_size_request(toolbar->btnNieuw, 70, -1);

    toolbar->btnRefresh = gtk_button_new();
    vbox = gtk_vbox_new(FALSE, 0);
    label = gtk_label_new("Refresh");
    gtk_button_set_relief(GTK_BUTTON(toolbar->btnRefresh), GTK_RELIEF_NONE);
#if GTK_CHECK_VERSION(2,4,0)
    gtk_button_set_focus_on_click(GTK_BUTTON(toolbar->btnRefresh), FALSE);
#endif

    pixbuf = ftd_gui_main_get_pixbuf("icon-refresh_off");
    toolbar->imgRefresh = gtk_image_new_from_pixbuf(pixbuf);
    gtk_box_pack_start(GTK_BOX(vbox), toolbar->imgRefresh, TRUE, TRUE, 0);

    gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
    gtk_container_add(GTK_CONTAINER(toolbar->btnRefresh), vbox);
    gtk_container_add(GTK_CONTAINER(toolbar->widget), toolbar->btnRefresh);
    gtk_widget_set_size_request(toolbar->btnRefresh, 70, -1);

    toolbar->btnDayselector = gtk_button_new();
    vbox = gtk_vbox_new(FALSE, 0);
    label = gtk_label_new("Day selector");
    gtk_button_set_relief(GTK_BUTTON(toolbar->btnDayselector), GTK_RELIEF_NONE);
#if GTK_CHECK_VERSION(2,4,0)
    gtk_button_set_focus_on_click(GTK_BUTTON(toolbar->btnDayselector), FALSE);
#endif
    pixbuf = ftd_gui_main_get_pixbuf("icon-dayselector_off");
    toolbar->imgDayselector = gtk_image_new_from_pixbuf(pixbuf);
    gtk_box_pack_start(GTK_BOX(vbox), toolbar->imgDayselector, TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
    gtk_container_add(GTK_CONTAINER(toolbar->btnDayselector), vbox);
    gtk_container_add(GTK_CONTAINER(toolbar->widget), toolbar->btnDayselector);

    toolbar->btnVrienden = gtk_button_new();
    vbox = gtk_vbox_new(FALSE, 0);
    label = gtk_label_new("Vrienden van FTD");
    gtk_button_set_relief(GTK_BUTTON(toolbar->btnVrienden), GTK_RELIEF_NONE);
#if GTK_CHECK_VERSION(2,4,0)
    gtk_button_set_focus_on_click(GTK_BUTTON(toolbar->btnVrienden), FALSE);
#endif
    pixbuf = ftd_gui_main_get_pixbuf("icon-vrienden_off");
    toolbar->imgVrienden = gtk_image_new_from_pixbuf(pixbuf);
    gtk_box_pack_start(GTK_BOX(vbox), toolbar->imgVrienden, TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
    gtk_container_add(GTK_CONTAINER(toolbar->btnVrienden), vbox);
    gtk_container_add(GTK_CONTAINER(toolbar->widget), toolbar->btnVrienden);

    toolbar->btnWatchlist = gtk_button_new();
    vbox = gtk_vbox_new(FALSE, 0);
    label = gtk_label_new("Watchlist");
    gtk_button_set_relief(GTK_BUTTON(toolbar->btnWatchlist), GTK_RELIEF_NONE);
#if GTK_CHECK_VERSION(2,4,0)
    gtk_button_set_focus_on_click(GTK_BUTTON(toolbar->btnWatchlist), FALSE);
#endif
    pixbuf = ftd_gui_main_get_pixbuf("icon-watchlist_off");
    toolbar->imgWatchlist = gtk_image_new_from_pixbuf(pixbuf);
    gtk_box_pack_start(GTK_BOX(vbox), toolbar->imgWatchlist, TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
    gtk_container_add(GTK_CONTAINER(toolbar->btnWatchlist), vbox);
    gtk_container_add(GTK_CONTAINER(toolbar->widget), toolbar->btnWatchlist);

    toolbar->btnWatcher = gtk_button_new();
    vbox = gtk_vbox_new(FALSE, 0);
    label = gtk_label_new("Watcher");
    gtk_button_set_relief(GTK_BUTTON(toolbar->btnWatcher), GTK_RELIEF_NONE);
#if GTK_CHECK_VERSION(2,4,0)
    gtk_button_set_focus_on_click(GTK_BUTTON(toolbar->btnWatcher), FALSE);
#endif
    pixbuf = ftd_gui_main_get_pixbuf("icon-watchlist_off"); // FIXME: Moet apart icoontje worden
    toolbar->imgWatcher = gtk_image_new_from_pixbuf(pixbuf);
    gtk_box_pack_start(GTK_BOX(vbox), toolbar->imgWatcher, TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
    gtk_container_add(GTK_CONTAINER(toolbar->btnWatcher), vbox);
    gtk_container_add(GTK_CONTAINER(toolbar->widget), toolbar->btnWatcher);

    toolbar->btnMyFTD = gtk_button_new();
    vbox = gtk_vbox_new(FALSE, 0);
    label = gtk_label_new("MyFTD");
    gtk_button_set_relief(GTK_BUTTON(toolbar->btnMyFTD), GTK_RELIEF_NONE);
#if GTK_CHECK_VERSION(2,4,0)
    gtk_button_set_focus_on_click(GTK_BUTTON(toolbar->btnMyFTD), FALSE);
#endif
    pixbuf = ftd_gui_main_get_pixbuf("icon-myftd_off");
    toolbar->imgMyFTD = gtk_image_new_from_pixbuf(pixbuf);
    gtk_box_pack_start(GTK_BOX(vbox), toolbar->imgMyFTD, TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
    gtk_container_add(GTK_CONTAINER(toolbar->btnMyFTD), vbox);
    gtk_container_add(GTK_CONTAINER(toolbar->widget), toolbar->btnMyFTD);

    separator2 = gtk_vseparator_new();
    gtk_widget_set_size_request(separator2, 15 , 65);
    gtk_container_add(GTK_CONTAINER(gui->toolbar->widget), separator2);

    toolbar->btnHulp = gtk_button_new();
    vbox = gtk_vbox_new(FALSE, 0);
    label = gtk_label_new("Hulp");
    gtk_button_set_relief(GTK_BUTTON(toolbar->btnHulp), GTK_RELIEF_NONE);
#if GTK_CHECK_VERSION(2,4,0)
    gtk_button_set_focus_on_click(GTK_BUTTON(toolbar->btnHulp), FALSE);
#endif
    pixbuf = ftd_gui_main_get_pixbuf("icon-hulp_off");
    toolbar->imgHulp = gtk_image_new_from_pixbuf(pixbuf);
    gtk_box_pack_start(GTK_BOX(vbox), toolbar->imgHulp, TRUE, TRUE, 0);
    gtk_box_pack_end(GTK_BOX(vbox), label, FALSE, FALSE, 0);
    gtk_container_add(GTK_CONTAINER(toolbar->btnHulp), vbox);
    gtk_container_add(GTK_CONTAINER(toolbar->widget), toolbar->btnHulp);

    gtk_widget_show_all(toolbar->widget);

    gtk_widget_size_request(toolbar->btnDayselector, &req);
    // Lelijke workaround om te voorkomen dat de 'Day Selector' knop ingekrompen wordt
    req.width += 5;

    gtk_widget_set_size_request(toolbar->btnUpdaten, req.width, -1);
    gtk_widget_set_size_request(toolbar->btnConfig, req.width, -1);
    gtk_widget_set_size_request(toolbar->btnToevoegen, req.width, -1);
    gtk_widget_set_size_request(toolbar->btnMail, req.width, -1);
    gtk_widget_set_size_request(toolbar->btnRefresh, req.width, -1);
    gtk_widget_set_size_request(toolbar->btnNieuw, req.width, -1);
    gtk_widget_set_size_request(toolbar->btnDayselector, req.width, -1);
    gtk_widget_set_size_request(toolbar->btnWatchlist, req.width, -1);
    gtk_widget_set_size_request(toolbar->btnWatcher, req.width, -1);
    gtk_widget_set_size_request(toolbar->btnMyFTD, req.width, -1);
    gtk_widget_set_size_request(toolbar->btnHulp, req.width, -1);

    /* updaten */
    g_signal_connect((gpointer) toolbar->btnUpdaten, "clicked",
                     G_CALLBACK(ftd_gui_callback_updaten), NULL);
    g_signal_connect((gpointer) toolbar->btnUpdaten, "enter",
                     G_CALLBACK(ftd_gui_toolbar_updaten_enter), NULL);
    g_signal_connect((gpointer) toolbar->btnUpdaten, "leave",
                     G_CALLBACK(ftd_gui_toolbar_updaten_leave), NULL);

    /* config */
    g_signal_connect((gpointer) toolbar->btnConfig, "clicked",
                     G_CALLBACK(ftd_gui_callback_config), NULL);
    g_signal_connect((gpointer) toolbar->btnConfig, "enter",
                     G_CALLBACK(ftd_gui_toolbar_config_enter), NULL);
    g_signal_connect((gpointer) toolbar->btnConfig, "leave",
                     G_CALLBACK(ftd_gui_toolbar_config_leave), NULL);

    /* toevoegen */
    g_signal_connect((gpointer) toolbar->btnToevoegen, "clicked",
                     G_CALLBACK(ftd_gui_callback_toevoegen), NULL);
    g_signal_connect((gpointer) toolbar->btnToevoegen, "enter",
                     G_CALLBACK(ftd_gui_toolbar_toevoegen_enter), NULL);
    g_signal_connect((gpointer) toolbar->btnToevoegen, "leave",
                     G_CALLBACK(ftd_gui_toolbar_toevoegen_leave), NULL);

    /* FTD Mail */
    g_signal_connect((gpointer) toolbar->btnMail, "clicked",
                     G_CALLBACK(ftd_gui_callback_mail), NULL);
    g_signal_connect((gpointer) toolbar->btnMail, "enter",
                     G_CALLBACK(ftd_gui_toolbar_mail_enter), NULL);
    g_signal_connect((gpointer) toolbar->btnMail, "leave",
                     G_CALLBACK(ftd_gui_toolbar_mail_leave), NULL);

    /* Nieuwe FTD Postings */
    g_signal_connect((gpointer) toolbar->btnNieuw, "clicked",
                     G_CALLBACK(ftd_gui_callback_nieuw), NULL);
    g_signal_connect((gpointer) toolbar->btnNieuw, "enter",
                     G_CALLBACK(ftd_gui_toolbar_nieuw_enter), NULL);
    g_signal_connect((gpointer) toolbar->btnNieuw, "leave",
                     G_CALLBACK(ftd_gui_toolbar_nieuw_leave), NULL);

    /* Refresh */
    g_signal_connect((gpointer) toolbar->btnRefresh, "clicked",
                     G_CALLBACK(ftd_gui_callback_refresh), NULL);
    g_signal_connect((gpointer) toolbar->btnRefresh, "enter",
                     G_CALLBACK(ftd_gui_toolbar_refresh_enter), NULL);
    g_signal_connect((gpointer) toolbar->btnRefresh, "leave",
                     G_CALLBACK(ftd_gui_toolbar_refresh_leave), NULL);

    /* Day Selector */
    g_signal_connect((gpointer) toolbar->btnDayselector, "clicked",
                     G_CALLBACK(ftd_gui_callback_dayselector), NULL);
    g_signal_connect((gpointer) toolbar->btnDayselector, "enter",
                     G_CALLBACK(ftd_gui_toolbar_dayselector_enter), NULL);
    g_signal_connect((gpointer) toolbar->btnDayselector, "leave",
                     G_CALLBACK(ftd_gui_toolbar_dayselector_leave), NULL);

    /* Vrienden van FTD */
    g_signal_connect((gpointer) toolbar->btnVrienden, "clicked",
                     G_CALLBACK(ftd_gui_callback_vrienden), NULL);
    g_signal_connect((gpointer) toolbar->btnVrienden, "enter",
                     G_CALLBACK(ftd_gui_toolbar_vrienden_enter), NULL);
    g_signal_connect((gpointer) toolbar->btnVrienden, "leave",
                     G_CALLBACK(ftd_gui_toolbar_vrienden_leave), NULL);

    /* Watchlist */
    g_signal_connect((gpointer) toolbar->btnWatchlist, "clicked",
                     G_CALLBACK(ftd_gui_callback_watchlist), NULL);
    g_signal_connect((gpointer) toolbar->btnWatchlist, "enter",
                     G_CALLBACK(ftd_gui_toolbar_watchlist_enter), NULL);
    g_signal_connect((gpointer) toolbar->btnWatchlist, "leave",
                     G_CALLBACK(ftd_gui_toolbar_watchlist_leave), NULL);

    /* Watcher */
    g_signal_connect((gpointer) toolbar->btnWatcher, "clicked",
                     G_CALLBACK(ftd_gui_callback_watcher), NULL);
    g_signal_connect((gpointer) toolbar->btnWatcher, "enter",
                     G_CALLBACK(ftd_gui_toolbar_watcher_enter), NULL);
    g_signal_connect((gpointer) toolbar->btnWatcher, "leave",
                     G_CALLBACK(ftd_gui_toolbar_watcher_leave), NULL);

    /* MyFTD */
    g_signal_connect((gpointer) toolbar->btnMyFTD, "clicked",
                     G_CALLBACK(ftd_gui_callback_myftd), NULL);
    g_signal_connect((gpointer) toolbar->btnMyFTD, "enter",
                     G_CALLBACK(ftd_gui_toolbar_myftd_enter), NULL);
    g_signal_connect((gpointer) toolbar->btnMyFTD, "leave",
                     G_CALLBACK(ftd_gui_toolbar_myftd_leave), NULL);

    /* About */
    g_signal_connect((gpointer) toolbar->btnHulp, "clicked",
                     G_CALLBACK(ftd_gui_callback_about), NULL);
    g_signal_connect((gpointer) toolbar->btnHulp, "enter",
                     G_CALLBACK(ftd_gui_toolbar_hulp_enter), NULL);
    g_signal_connect((gpointer) toolbar->btnHulp, "leave",
                     G_CALLBACK(ftd_gui_toolbar_hulp_leave), NULL);
}

gboolean
ftd_gui_toolbar_adjust_watchlist(gpointer data)
{
    GdkPixbuf *pixbuf = NULL;
    int newstate = -1;

    g_assert(gui);

    if (ftd_watchlist_get_state() == 0) {
        newstate = 1;
        pixbuf = ftd_gui_main_get_pixbuf("icon-watchlist_active");
    }

    if (ftd_watchlist_get_state() == 1) {
        newstate = 0;
        pixbuf = ftd_gui_main_get_pixbuf("icon-watchlist_off");
    }

    if (ftd_watchlist_get_state() == 2 || newstate == -1) {

        /* Herstel het icoon naar de originele staat */
        pixbuf = ftd_gui_main_get_pixbuf("icon-watchlist_off");
        gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgWatchlist), pixbuf);

        if (ftd_watchlist_get_state() != 2) {
            g_warning("ftd_gui_toolbar_adjust_watchlist(): invalid state : %i\n", ftd_watchlist_get_state());
        }

        return FALSE;
    }

    ftd_watchlist_set_state(newstate);
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgWatchlist), pixbuf);

    /* Zolang we TRUE retourneren wordt dit stuk om de x ms herhaalt */
    return TRUE;
}

gboolean
ftd_gui_toolbar_adjust_watcher(gpointer data)
{
    GdkPixbuf *pixbuf = NULL;
    int newstate = -1;

    g_assert(gui);

    if (ftd_watcher_get_state() == 0) {
        newstate = 1;
        pixbuf = ftd_gui_main_get_pixbuf("icon-watchlist_active");  // FIXME: Moet nieuw icoontje worden
    }

    if (ftd_watcher_get_state() == 1) {
        newstate = 0;
        pixbuf = ftd_gui_main_get_pixbuf("icon-watchlist_off");     // FIXME: Moet nieuw icoontje worden
    }

    if (ftd_watcher_get_state() == 2 || newstate == -1) {

        /* Herstel het icoon naar de originele staat */
        pixbuf = ftd_gui_main_get_pixbuf("icon-watchlist_off"); // FIXME: Moet nieuw icoontje worden
        gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgWatcher), pixbuf);

        if (ftd_watcher_get_state() != 2) {
            g_warning("ftd_gui_toolbar_adjust_watcher(): invalid state : %i\n", ftd_watcher_get_state());
        }

        return FALSE;
    }

    ftd_watcher_set_state(newstate);
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgWatcher), pixbuf);

    /* Zolang we TRUE retourneren wordt dit stuk om de x ms herhaalt */
    return TRUE;
}

void
ftd_gui_toolbar_updaten_enter(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-updaten_on");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgUpdaten), pixbuf);
}

void
ftd_gui_toolbar_updaten_leave(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-updaten_off");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgUpdaten), pixbuf);
}

void
ftd_gui_toolbar_config_enter(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-config_on");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgConfig), pixbuf);
}

void
ftd_gui_toolbar_config_leave(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-config_off");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgConfig), pixbuf);
}

void
ftd_gui_toolbar_toevoegen_enter(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-toevoegen_on");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgToevoegen), pixbuf);
}

void
ftd_gui_toolbar_toevoegen_leave(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-toevoegen_off");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgToevoegen), pixbuf);
}

void
ftd_gui_toolbar_mail_enter(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-mail_on");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgMail), pixbuf);
}

void
ftd_gui_toolbar_mail_leave(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-mail_off");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgMail), pixbuf);
}

void
ftd_gui_toolbar_nieuw_enter(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-nieuw_on");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgNieuw), pixbuf);
}

void
ftd_gui_toolbar_nieuw_leave(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-nieuw_off");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgNieuw), pixbuf);
}

void
ftd_gui_toolbar_refresh_enter(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-refresh_on");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgRefresh), pixbuf);
}

void
ftd_gui_toolbar_refresh_leave(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-refresh_off");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgRefresh), pixbuf);
}

void
ftd_gui_toolbar_dayselector_enter(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-dayselector_on");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgDayselector), pixbuf);
}

void
ftd_gui_toolbar_dayselector_leave(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-dayselector_off");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgDayselector), pixbuf);
}

void
ftd_gui_toolbar_vrienden_enter(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-vrienden_on");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgVrienden), pixbuf);
}

void
ftd_gui_toolbar_vrienden_leave(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-vrienden_off");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgVrienden), pixbuf);
}

void
ftd_gui_toolbar_watchlist_enter(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-watchlist_on");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgWatchlist), pixbuf);
}

void
ftd_gui_toolbar_watchlist_leave(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-watchlist_off");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgWatchlist), pixbuf);
}

void
ftd_gui_toolbar_watcher_enter(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-watchlist_on");  // FIXME: Moet apart icoontje worden
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgWatcher), pixbuf);
}

void
ftd_gui_toolbar_watcher_leave(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-watchlist_off"); // FIXME: Moet apart icoontje worden
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgWatcher), pixbuf);
}

void
ftd_gui_toolbar_myftd_enter(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-myftd_on");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgMyFTD), pixbuf);
}

void
ftd_gui_toolbar_myftd_leave(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-myftd_off");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgMyFTD), pixbuf);
}

void
ftd_gui_toolbar_hulp_enter(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-hulp_on");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgHulp), pixbuf);
}

void
ftd_gui_toolbar_hulp_leave(GtkButton *sender, gpointer data)
{
    GdkPixbuf *pixbuf = NULL;

    g_assert(gui);

    pixbuf = ftd_gui_main_get_pixbuf("icon-hulp_off");
    gtk_image_set_from_pixbuf(GTK_IMAGE(gui->toolbar->imgHulp), pixbuf);
}
